@@ -1,15 +1,15 @@
 # version_dep.py
-"""Holds test code that is dependent of certain python versions"""
-# Copyright (c) 2009 Darcy Mason
+"""Holds test code that is dependent on certain python versions"""
+# Copyright (c) 2009-2012 Darcy Mason
 # This file is part of pydicom, released under a modified MIT license.
 #    See the file license.txt included with this distribution, also
 #    available at http://pydicom.googlecode.com
 
 import warnings
 
+
 def capture_warnings(function, *func_args, **func_kwargs):
-    """Capture and function result and warnings.
-    For python > 2.5
+    """Capture function result and warnings.
     """
     with warnings.catch_warnings(record=True) as w:
         warnings.simplefilter("always")
